import Vue from "vue";
import VueRouter from "vue-router";
import store from "@/store";

const originalPush = VueRouter.prototype.push;
VueRouter.prototype.push = function push(location) {
  return originalPush.call(this, location).catch(() => {});
};

Vue.use(VueRouter);

const recursion = function(paths, root, requireContext) {
  const dirs = paths.map((item) => item.split("/")[0]);
  const sets = Array.from(new Set(dirs));
  const router = sets.map((it) => {
    const reg = new RegExp("^" + it + "/");
    const _paths = paths
      .filter((item) => item !== it)
      .filter((item) => reg.test(item))
      .map((item) => item.substring(it.length + 1));
    const component = requireContext(root + it + "/index.vue").default;
    const children = recursion(_paths, root + it + "/", requireContext);
    const routerItem = {
      path: it,
      children,
      component,
      name: component.name || it,
      meta: component.meta || {},
      redirect: component.meta?.redirect,
    };
    if (!routerItem.children.length) delete routerItem.children;
    if (!routerItem.redirect === undefined) delete routerItem.redirect;
    return routerItem;
  });
  return router;
};

const genRouter = function(requireContext) {
  // 不读取components文件夹下的文件
  const paths = requireContext
    .keys()
    .map((item) => item.slice(2, -10))
    .filter((item) => !/components/.test(item));
  const router = recursion(paths, "./", requireContext).map((item) => ({
    ...item,
    path: "/" + item.path,
  }));
  return router;
};

const mapSort = function(menus) {
  // console.log(menus);
  //递归，对路由进行排序，添加name和icon
  const _menus = menus
    .sort((a, b) => {
      return a.meta.sort - b.meta.sort;
    })
    .filter((item) => !item.meta.hideMenu);

  _menus.forEach((element) => {
    if (element.children && element.children.length) {
      element.children = element.children.filter((item) => !item.meta.hideMenu);
      if (element.children.length) {
        element.children = mapSort(element.children);
      } else {
        delete element.children;
      }
    }
  });
  return _menus;
};
const routes = [
  {
    path: "",
    redirect: "/login",
  },
  {
    path: "/login",
    name: "login",
    component: () =>
      import(/* webpackChunkName: "accout" */ "@/views/login.vue"),
    // redirect: "/index"
  },
];

const requireContext = require.context(
  "@/views/",
  true,
  /^\.\/.*\/index\.vue$/
);

const asyncRoutes = genRouter(requireContext); //
const asyncRoutesSort = mapSort(asyncRoutes); //筛选排序后的路由

const router = new VueRouter({
  routes: routes,
  base: process.env.BASE_URL,
});

router.beforeEach((to, from, next) => {
  let storeAuth = localStorage.getItem("userAuth");
  let storeUserInfo = localStorage.getItem("userInfo");

  store.dispatch("user/setUserAuth", JSON.parse(storeAuth));
  store.dispatch("user/setUserInfo", JSON.parse(storeUserInfo));

  let userAuth = store.state.user.userAuth;
  // console.log(to, userAuth);
  if (to.meta.keepAlive) {
    store.dispatch("app/addCacheRoutes", to.name);
  }
  if (
    (userAuth && userAuth.includes(to.matched[0]?.name)) ||
    ["login"].includes(to.matched[0]?.name)
  ) {
    next();
  } else {
    router.replace("/login");
    next();
  }
});

router.addRoutes(asyncRoutes);
// console.log(router);
export default router;
export { asyncRoutes, asyncRoutesSort };

// this.$store.dispatch("app/removeCacheRoutes", findItem.parent.name);
